@@ -13,17 +13,7 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
-
-    Change History (most recent first):
-
-$Log: stdafx.h,v $
-Revision 1.2  2006/08/14 23:23:57  cheshire
-Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
-
-Revision 1.1  2005/05/20 22:01:02  bradley
-Bonjour for Windows example code to browse for HTTP services and deliver via Window messages.
-
-*/
+ */
 
 // Standard Windows pre-compiled header file.
 
