@@ -13,107 +13,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
-
-    Change History (most recent first):
-
-$Log: mDNSMacOS9.c,v $
-Revision 1.46  2006/08/14 23:24:29  cheshire
-Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
-
-Revision 1.45  2006/03/19 02:00:14  cheshire
-<rdar://problem/4073825> Improve logic for delaying packets after repeated interface transitions
-
-Revision 1.44  2005/09/16 21:06:50  cheshire
-Use mDNS_TimeNow_NoLock macro, instead of writing "mDNSPlatformRawTime() + m->timenow_adjust" all over the place
-
-Revision 1.43  2004/12/17 23:37:49  cheshire
-<rdar://problem/3485365> Guard against repeating wireless dissociation/re-association
-(and other repetitive configuration changes)
-
-Revision 1.42  2004/12/16 20:43:39  cheshire
-interfaceinfo.fMask should be interfaceinfo.fNetmask
-
-Revision 1.41  2004/10/16 00:17:00  cheshire
-<rdar://problem/3770558> Replace IP TTL 255 check with local subnet source address check
-
-Revision 1.40  2004/09/27 23:56:27  cheshire
-Fix infinite loop where mDNSPlatformUnlock() called mDNS_TimeNow(),
-and then mDNS_TimeNow() called mDNSPlatformUnlock()
-
-Revision 1.39  2004/09/21 21:02:54  cheshire
-Set up ifname before calling mDNS_RegisterInterface()
-
-Revision 1.38  2004/09/17 01:08:50  cheshire
-Renamed mDNSClientAPI.h to mDNSEmbeddedAPI.h
-  The name "mDNSClientAPI.h" is misleading to new developers looking at this code. The interfaces
-  declared in that file are ONLY appropriate to single-address-space embedded applications.
-  For clients on general-purpose computers, the interfaces defined in dns_sd.h should be used.
-
-Revision 1.37  2004/09/17 00:19:10  cheshire
-For consistency with AllDNSLinkGroupv6, rename AllDNSLinkGroup to AllDNSLinkGroupv4
-
-Revision 1.36  2004/09/16 21:59:16  cheshire
-For consistency with zerov6Addr, rename zeroIPAddr to zerov4Addr
-
-Revision 1.35  2004/09/16 00:24:49  cheshire
-<rdar://problem/3803162> Fix unsafe use of mDNSPlatformTimeNow()
-
-Revision 1.34  2004/09/14 23:42:36  cheshire
-<rdar://problem/3801296> Need to seed random number generator from platform-layer data
-
-Revision 1.33  2004/09/14 23:16:31  cheshire
-Fix compile error: mDNS_SetFQDNs has been renamed to mDNS_SetFQDN
-
-Revision 1.32  2004/09/14 21:03:16  cheshire
-Fix spacing
-
-Revision 1.31  2004/08/14 03:22:42  cheshire
-<rdar://problem/3762579> Dynamic DNS UI <-> mDNSResponder glue
-Add GetUserSpecifiedDDNSName() routine
-Convert ServiceRegDomain to domainname instead of C string
-Replace mDNS_GenerateFQDN/mDNS_GenerateGlobalFQDN with mDNS_SetFQDNs
-
-Revision 1.30  2004/07/29 19:26:03  ksekar
-Plaform-level changes for NATPMP support
-
-Revision 1.29  2004/05/26 20:53:16  cheshire
-Remove unncecessary "return( -1 );" at the end of mDNSPlatformUTC()
-
-Revision 1.28  2004/05/20 18:39:06  cheshire
-Fix build broken by addition of mDNSPlatformUTC requirement
-
-Revision 1.27  2004/04/21 02:49:11  cheshire
-To reduce future confusion, renamed 'TxAndRx' to 'McastTxRx'
-
-Revision 1.26  2004/04/09 17:43:03  cheshire
-Make sure to set the McastTxRx field so that duplicate suppression works correctly
-
-Revision 1.25  2004/03/15 18:55:38  cheshire
-Comment out debugging message
-
-Revision 1.24  2004/03/12 21:30:26  cheshire
-Build a System-Context Shared Library from mDNSCore, for the benefit of developers
-like Muse Research who want to be able to use mDNS/DNS-SD from GPL-licensed code.
-
-Revision 1.23  2004/02/09 23:24:43  cheshire
-Need to set TTL 255 to interoperate with peers that check TTL (oops!)
-
-Revision 1.22  2004/01/27 20:15:23  cheshire
-<rdar://problem/3541288>: Time to prune obsolete code for listening on port 53
-
-Revision 1.21  2004/01/24 04:59:16  cheshire
-Fixes so that Posix/Linux, OS9, Windows, and VxWorks targets build again
-
-Revision 1.20  2003/11/14 20:59:09  cheshire
-Clients can't use AssignDomainName macro because mDNSPlatformMemCopy is defined in mDNSPlatformFunctions.h.
-Best solution is just to combine mDNSEmbeddedAPI.h and mDNSPlatformFunctions.h into a single file.
-
-Revision 1.19  2003/08/18 23:09:20  cheshire
-<rdar://problem/3382647> mDNSResponder divide by zero in mDNSPlatformRawTime()
-
-Revision 1.18  2003/08/12 19:56:24  cheshire
-Update to APSL 2.0
-
  */
 
 #include <stdio.h>
@@ -278,7 +177,7 @@ mDNSlocal OSStatus readpacket(mDNS *m)
 
 	#if OTCARBONAPPLICATION
 	// IP_RCVDSTADDR is known to fail on OS X Carbon, so we'll just assume the packet was probably multicast
-	destaddr.ip.v4  = AllDNSLinkGroupv4;
+	destaddr.ip.v4  = AllDNSLinkGroup_v4.ip.v4;
 	#endif
 
 	if (recvdata.opt.len)
@@ -289,7 +188,7 @@ mDNSlocal OSStatus readpacket(mDNS *m)
 			err = OTNextOption(recvdata.opt.buf, recvdata.opt.len, &c);
 			if (err || !c) break;
 			if (c->level == INET_IP && c->name == IP_RCVDSTADDR && c->len - kOTOptionHeaderSize == sizeof(destaddr.ip.v4))
-				mDNSPlatformMemCopy(c->value, &destaddr.ip.v4, sizeof(destaddr.ip.v4));
+				mDNSPlatformMemCopy(&destaddr.ip.v4, c->value, sizeof(destaddr.ip.v4));
 			}
 		}
 
@@ -302,39 +201,73 @@ mDNSlocal OSStatus readpacket(mDNS *m)
 	return(err);
 	}
 
-mDNSexport mStatus mDNSPlatformTCPConnect(const mDNSAddr *dst, mDNSOpaque16 dstport, mDNSInterfaceID InterfaceID,
-										  TCPConnectionCallback callback, void *context, int *descriptor)
+mDNSexport TCPSocket *mDNSPlatformTCPSocket(mDNS * const m, TCPSocketFlags flags, mDNSIPPort * port)
 	{
+	(void)m;			// Unused
+	(void)flags;		// Unused
+	(void)port;			// Unused
+	return NULL;
+	}
+
+mDNSexport TCPSocket *mDNSPlatformTCPAccept(TCPSocketFlags flags, int sd)
+	{
+	(void)flags;		// Unused
+	(void)sd;			// Unused
+	return NULL;
+	}
+
+mDNSexport int mDNSPlatformTCPGetFD(TCPSocket *sock)
+	{
+	(void)sock;			// Unused
+	return -1;
+	}
+
+mDNSexport mStatus mDNSPlatformTCPConnect(TCPSocket *sock, const mDNSAddr * dst, mDNSOpaque16 dstport, mDNSInterfaceID InterfaceID,
+                                      TCPConnectionCallback callback, void * context)
+	{
+	(void)sock;			// Unused
 	(void)dst;			// Unused
 	(void)dstport;		// Unused
 	(void)InterfaceID;	// Unused
 	(void)callback;		// Unused
 	(void)context;		// Unused
-	(void)descriptor;	// Unused
 	return(mStatus_UnsupportedErr);
 	}
 
-mDNSexport void mDNSPlatformTCPCloseConnection(int sd)
+mDNSexport void mDNSPlatformTCPCloseConnection(TCPSocket *sd)
 	{
 	(void)sd;			// Unused
 	}
 
-mDNSexport int mDNSPlatformReadTCP(int sd, void *buf, int buflen)
+mDNSexport long mDNSPlatformReadTCP(TCPSocket *sock, void *buf, unsigned long buflen, mDNSBool *closed)
 	{
-	(void)sd;			// Unused
+	(void)sock;			// Unused
 	(void)buf;			// Unused
 	(void)buflen;		// Unused
+	(void)closed;		// Unused
 	return(0);
 	}
 
-mDNSexport int mDNSPlatformWriteTCP(int sd, const char *msg, int len)
+mDNSexport long mDNSPlatformWriteTCP(TCPSocket *sock, const char *msg, unsigned long len)
 	{
-	(void)sd;			// Unused
+	(void)sock;			// Unused
 	(void)msg;			// Unused
 	(void)len;			// Unused
 	return(0);
 	}
 
+mDNSexport UDPSocket *mDNSPlatformUDPSocket(mDNS * const m, mDNSIPPort port)
+	{
+	(void)m;			// Unused
+	(void)port;			// Unused
+	return NULL;
+	}
+
+mDNSexport void           mDNSPlatformUDPClose(UDPSocket *sock)
+	{
+	(void)sock;			// Unused
+	}
+
 mDNSlocal void mDNSOptionManagement(mDNS *const m)
 	{
 	OSStatus err;
@@ -727,11 +660,11 @@ mDNSexport void    mDNSPlatformUnlock(const mDNS *const m)
 	else OTLeaveNotifier(m->p->ep);
 	}
 
-mDNSexport void     mDNSPlatformStrCopy(const void *src,       void *dst)             { OTStrCopy((char*)dst, (char*)src); }
-mDNSexport UInt32   mDNSPlatformStrLen (const void *src)                              { return(OTStrLength((char*)src)); }
-mDNSexport void     mDNSPlatformMemCopy(const void *src,       void *dst, UInt32 len) { OTMemcpy(dst, src, len); }
-mDNSexport mDNSBool mDNSPlatformMemSame(const void *src, const void *dst, UInt32 len) { return(OTMemcmp(dst, src, len)); }
-mDNSexport void     mDNSPlatformMemZero(                       void *dst, UInt32 len) { OTMemzero(dst, len); }
+mDNSexport void     mDNSPlatformStrCopy(      void *dst, const void *src)             { OTStrCopy((char*)dst, (char*)src); }
+mDNSexport UInt32   mDNSPlatformStrLen (                 const void *src)             { return(OTStrLength((char*)src)); }
+mDNSexport void     mDNSPlatformMemCopy(      void *dst, const void *src, UInt32 len) { OTMemcpy(dst, src, len); }
+mDNSexport mDNSBool mDNSPlatformMemSame(const void *dst, const void *src, UInt32 len) { return(OTMemcmp(dst, src, len)); }
+mDNSexport void     mDNSPlatformMemZero(      void *dst,                  UInt32 len) { OTMemzero(dst, len); }
 mDNSexport void *   mDNSPlatformMemAllocate(mDNSu32 len)                              { return(OTAllocMem(len)); }
 mDNSexport void     mDNSPlatformMemFree(void *mem)                                    { OTFreeMem(mem); }
 mDNSexport mDNSu32  mDNSPlatformRandomSeed(void)                                      { return(TickCount()); }
