@@ -39,90 +39,43 @@
     OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
     (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
     ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-    Change History (most recent first):
-
-$Log: DNSServiceDiscoveryPref.m,v $
-Revision 1.8  2006/08/14 23:15:47  cheshire
-Tidy up Change History comment
-
-Revision 1.7  2006/07/14 03:59:14  cheshire
-Fix compile warnings: 'sortUsingFunction:context:' comparison function needs to return int
-
-Revision 1.6  2005/02/26 00:44:24  cheshire
-Restore default reg domain if user deletes text and clicks "apply"
-
-Revision 1.5  2005/02/25 02:29:28  cheshire
-Show yellow dot for "update in progress"
-
-Revision 1.4  2005/02/16 00:18:33  cheshire
-Bunch o' fixes
-
-Revision 1.3  2005/02/10 22:35:20  cheshire
-<rdar://problem/3727944> Update name
-
-Revision 1.2  2005/02/08 01:32:05  cheshire
-Add trimCharactersFromDomain routine to strip leading and trailing
-white space and punctuation from user-entered fields.
-
-Revision 1.1  2005/02/05 01:59:19  cheshire
-Add Preference Pane to facilitate testing of DDNS & wide-area features
-
-*/
+ */
 
 #import "DNSServiceDiscoveryPref.h"
 #import "ConfigurationAuthority.h"
 #import "PrivilegedOperations.h"
 #import <unistd.h>
 
+#include "../../Clients/ClientCommon.h"
+
+#ifndef NSINTEGER_DEFINED
+#define NSInteger int
+#endif
+
 @implementation DNSServiceDiscoveryPref
 
-static int
+static NSComparisonResult
 MyArrayCompareFunction(id val1, id val2, void *context)
 {
+	(void)context; // Unused
     return CFStringCompare((CFStringRef)val1, (CFStringRef)val2, kCFCompareCaseInsensitive);
 }
 
 
-static int
+static NSComparisonResult
 MyDomainArrayCompareFunction(id val1, id val2, void *context)
 {
+	(void)context; // Unused
 	NSString *domain1 = [val1 objectForKey:(NSString *)SC_DYNDNS_DOMAIN_KEY];
 	NSString *domain2 = [val2 objectForKey:(NSString *)SC_DYNDNS_DOMAIN_KEY];
     return CFStringCompare((CFStringRef)domain1, (CFStringRef)domain2, kCFCompareCaseInsensitive);
 }
 
 
-static const char *
-GetNextLabel(const char *cstr, char label[64])
-{
-	char *ptr = label;
-	while (*cstr && *cstr != '.')								// While we have characters in the label...
-		{
-		char c = *cstr++;
-		if (c == '\\')
-			{
-			c = *cstr++;
-			if (isdigit(cstr[-1]) && isdigit(cstr[0]) && isdigit(cstr[1]))
-				{
-				int v0 = cstr[-1] - '0';						// then interpret as three-digit decimal
-				int v1 = cstr[ 0] - '0';
-				int v2 = cstr[ 1] - '0';
-				int val = v0 * 100 + v1 * 10 + v2;
-				if (val <= 255) { c = (char)val; cstr += 2; }	// If valid three-digit decimal value, use it
-				}
-			}
-		*ptr++ = c;
-		if (ptr >= label+64) return(NULL);
-		}
-	if (*cstr) cstr++;											// Skip over the trailing dot (if present)
-	*ptr++ = 0;
-	return(cstr);
-}
-
-
 static void NetworkChanged(SCDynamicStoreRef store, CFArrayRef changedKeys, void *context)
 {
+	(void)store; // Unused
+	(void)changedKeys; // Unused
     DNSServiceDiscoveryPref * me = (DNSServiceDiscoveryPref *)context;
     assert(me != NULL);
     
@@ -133,6 +86,9 @@ static void NetworkChanged(SCDynamicStoreRef store, CFArrayRef changedKeys, void
 static void ServiceDomainEnumReply( DNSServiceRef sdRef, DNSServiceFlags flags, uint32_t interfaceIndex,
     DNSServiceErrorType errorCode, const char *replyDomain, void *context, DNSServiceFlags enumType)
 {    
+	(void)sdRef; // Unused
+	(void)interfaceIndex; // Unused
+	(void)errorCode; // Unused
     if (strcmp(replyDomain, "local.") == 0) return;  // local domain is not interesting
 
 	DNSServiceDiscoveryPref * me = (DNSServiceDiscoveryPref *)context;
@@ -186,7 +142,7 @@ static void ServiceDomainEnumReply( DNSServiceRef sdRef, DNSServiceFlags flags,
 }
 
 
-void
+static void
 browseDomainReply(DNSServiceRef sdRef, DNSServiceFlags flags, uint32_t interfaceIndex,
     DNSServiceErrorType errorCode, const char *replyDomain, void *context)
 {
@@ -194,7 +150,7 @@ browseDomainReply(DNSServiceRef sdRef, DNSServiceFlags flags, uint32_t interface
 }
 
 
-void
+static void
 registrationDomainReply(DNSServiceRef sdRef, DNSServiceFlags flags, uint32_t interfaceIndex,
     DNSServiceErrorType errorCode, const char *replyDomain, void *context)
 {
@@ -249,7 +205,7 @@ MySocketReadCallback(CFSocketRef s, CFSocketCallBackType type, CFDataRef address
 
 
 
-void
+static void
 MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 {
     CFSocketNativeHandle sock;
@@ -396,7 +352,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 	
 	if ([defaultBrowseDomainsArray count] > 0) {
 		NSEnumerator * arrayEnumerator = [defaultBrowseDomainsArray objectEnumerator];
-		while (domain = [arrayEnumerator nextObject]) {
+		while ((domain = [arrayEnumerator nextObject]) != NULL) {
 			if ([self domainAlreadyInList:domain] == NO) break;
 		}
 	}
@@ -419,6 +375,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (IBAction)removeBrowseDomainClicked:(id)sender;
 {
+	(void)sender; // Unused
 	int selectedBrowseDomain = [browseDomainList selectedRow];
 	[browseDomainsArray removeObjectAtIndex:selectedBrowseDomain];
 	[browseDomainList reloadData];
@@ -442,8 +399,9 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 
 
-- (int)numberOfRowsInTableView:(NSTableView *)tableView;
+- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView;
 {
+	(void)tableView; // Unused
 	int numberOfRows = 0;
 		
 	if (browseDomainsArray) {
@@ -453,15 +411,18 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 }
 
 
-- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem;
+- (void)tabView:(NSTabView *)xtabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem;
 {
+	(void)xtabView; // Unused
+	(void)tabViewItem; // Unused
 	[browseDomainList deselectAll:self];
 	[mainWindow makeFirstResponder:nil];
 }
  
 
-- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row;
+- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row;
 {
+	(void)tableView; // Unused
 	NSDictionary *browseDomainDict;
 	id           value = nil;
 		
@@ -573,12 +534,10 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
     [self startDomainBrowsing];
     [self watchForPreferenceChanges];
 	
-	[tabView setDelegate:self];    
-    
     InitConfigAuthority();
     err = EnsureToolInstalled();
     if (err == noErr) toolInstalled = YES;
-    else fprintf(stderr, "EnsureToolInstalled returned %ld\n", err);
+    else { long int tmp = err; fprintf(stderr, "EnsureToolInstalled returned %ld\n", tmp); }
     
 }
 
@@ -618,7 +577,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 		NSDictionary *domainDict;
 		NSString     *domainName;
 		NSEnumerator *arrayEnumerator = [browseDomainsArray objectEnumerator];
-		while (domainDict = [arrayEnumerator nextObject]) {
+		while ((domainDict = [arrayEnumerator nextObject]) != NULL) {
 			domainName = [domainDict objectForKey:(NSString *)SC_DYNDNS_DOMAIN_KEY];
 			if ([domainString caseInsensitiveCompare:domainName] == NSOrderedSame) return YES;
 		}
@@ -637,6 +596,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (void)addBrowseDomainSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
 {
+	(void)contextInfo; // Unused
     [sheet orderOut:self];
     [self enableControls];
     
@@ -680,7 +640,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
     if ([sender isEqualTo:hostNameSharedSecretButton]) {		
         if (hostNameSharedSecretValue) {
 			[sharedSecretValue setStringValue:hostNameSharedSecretValue];
-        } else if (keyName = [self sharedSecretKeyName:[hostName stringValue]]) {
+        } else if ((keyName = [self sharedSecretKeyName:[hostName stringValue]]) != NULL) {
 			[sharedSecretName setStringValue:keyName];
             [sharedSecretValue setStringValue:@"****************"];
 		} else {
@@ -691,7 +651,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
     } else {        
         if (regSharedSecretValue) {
 			[sharedSecretValue setStringValue:regSharedSecretValue];
-        } else if (keyName = [self sharedSecretKeyName:[regDomainsComboBox stringValue]]) {
+        } else if ((keyName = [self sharedSecretKeyName:[regDomainsComboBox stringValue]]) != NULL) {
 			[sharedSecretName setStringValue:keyName];
             [sharedSecretValue setStringValue:@"****************"];
 		} else {
@@ -745,6 +705,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (void)controlTextDidChange:(NSNotification *)notification;
 {
+	(void)notification; // Unused
     [self updateApplyButtonState];
 }
 
@@ -752,15 +713,16 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (IBAction)comboAction:(id)sender;
 {
+	(void)sender; // Unused
     [self updateApplyButtonState];
 }
 
 
-- (id)comboBox:(NSComboBox *)aComboBox objectValueForItemAtIndex:(int)index
+- (id)comboBox:(NSComboBox *)aComboBox objectValueForItemAtIndex:(int)ind
 {
     NSString *domain = nil;
-    if      ([aComboBox isEqualTo:browseDomainsComboBox]) domain = [browseDataSource objectAtIndex:index];
-    else if ([aComboBox isEqualTo:regDomainsComboBox])    domain = [registrationDataSource objectAtIndex:index];
+    if      ([aComboBox isEqualTo:browseDomainsComboBox]) domain = [browseDataSource objectAtIndex:ind];
+    else if ([aComboBox isEqualTo:regDomainsComboBox])    domain = [registrationDataSource objectAtIndex:ind];
     return domain;
 }
 
@@ -856,6 +818,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (IBAction)applyClicked:(id)sender
 {
+	(void)sender; // Unused
     [self applyCurrentState];
 }
 
@@ -912,6 +875,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (void)savePanelWillClose:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
 {
+	(void)sheet; // Unused
     DNSServiceDiscoveryPref * me = (DNSServiceDiscoveryPref *)contextInfo;
     
     if (returnCode == NSAlertDefaultReturn) {
@@ -963,7 +927,7 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
         SecKeychainAttributeInfo attrInfo;
         SecKeychainAttributeList *attrList = NULL;
         SecKeychainAttribute attribute;
-		int i;
+		unsigned int i;
 		
         tags[0] = kSecAccountItemAttr;
         attrInfo.count = 1;
@@ -1169,8 +1133,10 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 }
 
 
-- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(int)row;
+- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(NSInteger)row;
 {
+	(void)row; // Unused
+	(void)tableView; // Unused
 	return browseDomainListEnabled;
 }
 
@@ -1198,13 +1164,32 @@ MyDNSServiceAddServiceToRunLoop(MyDNSServiceState * query)
 
 - (void)authorizationViewDidAuthorize:(SFAuthorizationView *)view
 {
+	(void)view; // Unused
     [self enableControls];
 }
 
 
 - (void)authorizationViewDidDeauthorize:(SFAuthorizationView *)view
 {    
+	(void)view; // Unused
     [self disableControls];
 }
 
 @end
+
+
+// Note: The C preprocessor stringify operator ('#') makes a string from its argument, without macro expansion
+// e.g. If "version" is #define'd to be "4", then STRINGIFY_AWE(version) will return the string "version", not "4"
+// To expand "version" to its value before making the string, use STRINGIFY(version) instead
+#define STRINGIFY_ARGUMENT_WITHOUT_EXPANSION(s) #s
+#define STRINGIFY(s) STRINGIFY_ARGUMENT_WITHOUT_EXPANSION(s)
+
+// NOT static -- otherwise the compiler may optimize it out
+// The "@(#) " pattern is a special prefix the "what" command looks for
+const char VersionString_SCCS[] = "@(#) Bonjour Preference Pane " STRINGIFY(mDNSResponderVersion) " (" __DATE__ " " __TIME__ ")";
+
+#if _BUILDING_XCODE_PROJECT_
+// If the process crashes, then this string will be magically included in the automatically-generated crash log
+const char *__crashreporter_info__ = VersionString_SCCS + 5;
+asm(".desc ___crashreporter_info__, 0x10");
+#endif
