@@ -39,30 +39,7 @@
     OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
     (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
     ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-    Change History (most recent first):
-
-$Log: PrivilegedOperations.c,v $
-Revision 1.6  2006/08/14 23:15:47  cheshire
-Tidy up Change History comment
-
-Revision 1.5  2006/06/10 02:07:11  mkrochma
-Whoa.  Make sure code compiles before checking it in.
-
-Revision 1.4  2006/05/27 02:32:38  mkrochma
-Wait for installer script to exit before returning result
-
-Revision 1.3  2005/06/04 04:50:00  cheshire
-<rdar://problem/4138070> ddnswriteconfig (Bonjour PreferencePane) vulnerability
-Use installtool instead of requiring ddnswriteconfig to self-install
-
-Revision 1.2  2005/02/10 22:35:20  cheshire
-<rdar://problem/3727944> Update name
-
-Revision 1.1  2005/02/05 01:59:19  cheshire
-Add Preference Pane to facilitate testing of DDNS & wide-area features
-
-*/
+ */
 
 #include "PrivilegedOperations.h"
 #include "ConfigurationAuthority.h"
@@ -78,7 +55,7 @@ Add Preference Pane to facilitate testing of DDNS & wide-area features
 
 Boolean	gToolApproved = false;
 
-pid_t	execTool(const char *args[])
+static pid_t	execTool(const char *args[])
 // fork/exec and return new pid
 {
 	pid_t	child;
@@ -143,7 +120,6 @@ OSStatus EnsureToolInstalled(void)
 			char *installerargs[] = { toolSourcePath, NULL };
 			err = AuthorizationExecuteWithPrivileges(authRef, toolInstallerPath, 0, installerargs, (FILE**) NULL);
 			if (err == noErr) {
-				int status;
 				int pid = wait(&status);
 				if (pid > 0 && WIFEXITED(status)) {
 					err = WEXITSTATUS(status);
@@ -154,7 +130,7 @@ OSStatus EnsureToolInstalled(void)
 					err = -1;
 				}
 			}
-			(void) AuthorizationFree(authRef, kAuthorizationFlagDestroyRights);
+			(void) AuthorizationFree(authRef, kAuthorizationFlagDefaults);
 		}
 	}
 
