@@ -26,128 +26,6 @@
  * thinking that variables x and y are both of type "char*" -- and anyone who doesn't
  * understand why variable y is not of type "char*" just proves the point that poor code
  * layout leads people to unfortunate misunderstandings about how the C language really works.)
-
-    Change History (most recent first):
-
-$Log: Identify.c,v $
-Revision 1.36  2006/08/14 23:24:46  cheshire
-Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
-
-Revision 1.35  2006/06/12 18:22:42  cheshire
-<rdar://problem/4580067> mDNSResponder building warnings under Red Hat 64-bit (LP64) Linux
-
-Revision 1.34  2004/12/16 20:17:11  cheshire
-<rdar://problem/3324626> Cache memory management improvements
-
-Revision 1.33  2004/11/30 22:37:00  cheshire
-Update copyright dates and add "Mode: C; tab-width: 4" headers
-
-Revision 1.32  2004/10/19 21:33:21  cheshire
-<rdar://problem/3844991> Cannot resolve non-local registrations using the mach API
-Added flag 'kDNSServiceFlagsForceMulticast'. Passing through an interface id for a unicast name
-doesn't force multicast unless you set this flag to indicate explicitly that this is what you want
-
-Revision 1.31  2004/10/16 00:17:00  cheshire
-<rdar://problem/3770558> Replace IP TTL 255 check with local subnet source address check
-
-Revision 1.30  2004/09/21 23:29:51  cheshire
-<rdar://problem/3680045> DNSServiceResolve should delay sending packets
-
-Revision 1.29  2004/09/17 01:08:53  cheshire
-Renamed mDNSClientAPI.h to mDNSEmbeddedAPI.h
-  The name "mDNSClientAPI.h" is misleading to new developers looking at this code. The interfaces
-  declared in that file are ONLY appropriate to single-address-space embedded applications.
-  For clients on general-purpose computers, the interfaces defined in dns_sd.h should be used.
-
-Revision 1.28  2004/09/17 00:31:52  cheshire
-For consistency with ipv6, renamed rdata field 'ip' to 'ipv4'
-
-Revision 1.27  2004/09/16 01:58:22  cheshire
-Fix compiler warnings
-
-Revision 1.26  2004/08/24 21:55:07  cheshire
-Don't try to build IPv6 code on systems that don't have IPv6
-
-Revision 1.25  2004/07/20 23:42:37  cheshire
-Update to use only "_services._dns-sd._udp.local." meta-query for service enumeration
-
-Revision 1.24  2004/06/15 02:39:47  cheshire
-When displaying error message, only show command name, not entire path
-
-Revision 1.23  2004/05/18 23:51:26  cheshire
-Tidy up all checkin comments to use consistent "<rdar://problem/xxxxxxx>" format for bug numbers
-
-Revision 1.22  2004/04/20 22:43:28  cheshire
-Use _services._dns-sd._udp query, as documented in
-<http://files.dns-sd.org/draft-cheshire-dnsext-dns-sd-02.txt>
-
-Revision 1.21  2004/01/28 21:38:57  cheshire
-Also ask target host for _services._mdns._udp.local. list
-
-Revision 1.20  2004/01/28 19:04:38  cheshire
-Fix Ctrl-C handling when multiple targets are specified
-
-Revision 1.19  2004/01/28 03:49:30  cheshire
-Enhanced mDNSIdentify to make use of new targeted-query capability
-
-Revision 1.18  2004/01/27 19:06:51  cheshire
-Remove workaround for WWDC 2003 bug; no one has run that buggy build for a long time
-
-Revision 1.17  2004/01/22 03:57:00  cheshire
-Use the new meta-interface mDNSInterface_ForceMCast. This restores mDNSIdentify's
-ability to use multicast queries with non-link-local target addresses, like 17.x.x.x.
-
-Revision 1.16  2004/01/22 00:03:32  cheshire
-Add while() loop so that a list of targets may be specified on the command line
-
-Revision 1.15  2004/01/21 21:55:06  cheshire
-Don't need to wait for timeout once we've got the information we wanted
-
-Revision 1.14  2003/12/17 00:51:22  cheshire
-Changed mDNSNetMonitor and mDNSIdentify to link the object files
-instead of #including the "DNSCommon.c" "uDNS.c" and source files
-
-Revision 1.13  2003/12/13 03:05:28  ksekar
-<rdar://problem/3192548>: DynDNS: Unicast query of service records
-
-Revision 1.12  2003/11/14 21:27:09  cheshire
-<rdar://problem/3484766>: Security: Crashing bug in mDNSResponder
-Fix code that should use buffer size MAX_ESCAPED_DOMAIN_NAME (1005) instead of 256-byte buffers.
-
-Revision 1.11  2003/10/30 19:26:38  cheshire
-Fix warnings on certain compilers
-
-Revision 1.10  2003/09/02 20:38:57  cheshire
-#include <signal.h> for Linux
-
-Revision 1.9  2003/08/14 23:57:46  cheshire
-Report if there is no answer at all from the target host
-
-Revision 1.8  2003/08/14 02:19:55  cheshire
-<rdar://problem/3375491> Split generic ResourceRecord type into two separate types: AuthRecord and CacheRecord
-
-Revision 1.7  2003/08/12 19:56:26  cheshire
-Update to APSL 2.0
-
-Revision 1.6  2003/08/06 01:46:18  cheshire
-Distinguish no answer from partial answer
-
-Revision 1.5  2003/08/05 23:56:26  cheshire
-Update code to compile with the new mDNSCoreReceive() function that requires a TTL
-(Right now mDNSPosix.c just reports 255 -- we should fix this)
-
-Revision 1.4  2003/08/04 17:24:48  cheshire
-Combine the three separate A/AAAA/HINFO queries into a single qtype "ANY" query
-
-Revision 1.3  2003/08/04 17:14:08  cheshire
-Do both AAAA queries in parallel
-
-Revision 1.2  2003/08/02 02:25:13  cheshire
-Multiple improvements: Now displays host's name, and all v4 and v6 addresses, as well as HINFO record
-
-Revision 1.1  2003/08/01 02:20:02  cheshire
-Add mDNSIdentify tool, used to discover what version of mDNSResponder a particular host is running
-
  */
 
 //*************************************************************************************************************
@@ -184,6 +62,7 @@ static mDNS mDNSStorage;       // mDNS core uses this to store its globals
 static mDNS_PlatformSupport PlatformStorage;  // Stores this platform's globals
 #define RR_CACHE_SIZE 500
 static CacheEntity gRRCache[RR_CACHE_SIZE];
+mDNSexport const char ProgramName[] = "mDNSIdentify";
 
 static volatile int StopNow;	// 0 means running, 1 means stop because we got an answer, 2 means stop because of Ctrl-C
 static volatile int NumAnswers, NumAddr, NumAAAA, NumHINFO;
@@ -225,7 +104,7 @@ mDNSexport void mDNSCoreReceive(mDNS *const m, DNSMessage *const msg, const mDNS
 	__MDNS__mDNSCoreReceive(m, msg, end, srcaddr, srcport, &AllDNSLinkGroup_v4, dstport, InterfaceID);
 	}
 
-static void NameCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, mDNSBool AddRecord)
+mDNSlocal void NameCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, QC_result AddRecord)
 	{
 	(void)m;		// Unused
 	(void)question;	// Unused
@@ -239,7 +118,7 @@ static void NameCallback(mDNS *const m, DNSQuestion *question, const ResourceRec
 		}
 	}
 
-static void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, mDNSBool AddRecord)
+mDNSlocal void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, QC_result AddRecord)
 	{
 	(void)m;		// Unused
 	(void)question;	// Unused
@@ -278,10 +157,10 @@ static void InfoCallback(mDNS *const m, DNSQuestion *question, const ResourceRec
 		}
 
 	// If we've got everything we're looking for, don't need to wait any more
-	if (NumHINFO && (NumAddr || NumAAAA)) StopNow = 1;
+	if (/*NumHINFO && */ (NumAddr || NumAAAA)) StopNow = 1;
 	}
 
-static void ServicesCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, mDNSBool AddRecord)
+mDNSlocal void ServicesCallback(mDNS *const m, DNSQuestion *question, const ResourceRecord *const answer, QC_result AddRecord)
 	{
 	(void)m;		// Unused
 	(void)question;	// Unused
@@ -292,13 +171,11 @@ static void ServicesCallback(mDNS *const m, DNSQuestion *question, const Resourc
 	if (answer->rrtype == kDNSType_PTR && mDNSSameAddress(&lastsrc, &target))
 		{
 		NumAnswers++;
-		NumAddr++;
 		mprintf("%##s %s %##s\n", answer->name->c, DNSTypeName(answer->rrtype), answer->rdata->u.name.c);
-		StopNow = 1;
 		}
 	}
 
-mDNSexport void WaitForAnswer(mDNS *const m, int seconds)
+mDNSlocal void WaitForAnswer(mDNS *const m, int seconds)
 	{
 	struct timeval end;
 	gettimeofday(&end, NULL);
@@ -315,7 +192,11 @@ mDNSexport void WaitForAnswer(mDNS *const m, int seconds)
 		FD_ZERO(&readfds);
 		gettimeofday(&now, NULL);
 		if (remain.tv_usec < now.tv_usec) { remain.tv_usec += 1000000; remain.tv_sec--; }
-		if (remain.tv_sec < now.tv_sec) return;
+		if (remain.tv_sec < now.tv_sec)
+			{
+			if (!NumAnswers) printf("No response after %d seconds\n", seconds);
+			return;
+			}
 		remain.tv_usec -= now.tv_usec;
 		remain.tv_sec  -= now.tv_sec;
 		mDNSPosixGetFDSet(m, &nfds, &readfds, &remain);
@@ -327,16 +208,18 @@ mDNSexport void WaitForAnswer(mDNS *const m, int seconds)
 
 mDNSlocal mStatus StartQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, const mDNSAddr *target, mDNSQuestionCallback callback)
 	{
+	lastsrc = zeroAddr;
 	if (qname) MakeDomainNameFromDNSNameString(&q->qname, qname);
+	q->InterfaceID      = mDNSInterface_Any;
 	q->Target           = target ? *target : zeroAddr;
 	q->TargetPort       = MulticastDNSPort;
 	q->TargetQID        = zeroID;
-	q->InterfaceID      = mDNSInterface_Any;
 	q->qtype            = qtype;
 	q->qclass           = kDNSClass_IN;
 	q->LongLived        = mDNSfalse;
-	q->ExpectUnique     = mDNStrue;
+	q->ExpectUnique     = mDNSfalse;	// Don't want to stop after the first response packet
 	q->ForceMCast       = mDNStrue;		// Query via multicast, even for apparently uDNS names like 1.1.1.17.in-addr.arpa.
+	q->ReturnIntermed   = mDNStrue;
 	q->QuestionCallback = callback;
 	q->QuestionContext  = NULL;
 
@@ -359,7 +242,7 @@ mDNSlocal void DoOneQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, const mDNS
 mDNSlocal int DoQuery(DNSQuestion *q, char *qname, mDNSu16 qtype, const mDNSAddr *target, mDNSQuestionCallback callback)
 	{
 	DoOneQuery(q, qname, qtype, target, callback);
-	if (StopNow == 0 && target && target->type)
+	if (StopNow == 0 && NumAnswers == 0 && target && target->type)
 		{
 		mprintf("%##s %s Trying multicast\n", q->qname.c, DNSTypeName(q->qtype));
 		DoOneQuery(q, qname, qtype, NULL, callback);
@@ -417,6 +300,7 @@ mDNSexport int main(int argc, char **argv)
 		if (inet_pton(AF_INET, arg, &s4) == 1)
 			{
 			mDNSu8 *p = (mDNSu8 *)&s4;
+			// Note: This is reverse order compared to a normal dotted-decimal IP address, so we can't use our customary "%.4a" format code
 			mDNS_snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d.in-addr.arpa.", p[3], p[2], p[1], p[0]);
 			printf("%s\n", buffer);
 			target.type = mDNSAddrType_IPv4;
@@ -439,7 +323,7 @@ mDNSexport int main(int argc, char **argv)
 				}
 			mDNS_snprintf(&buffer[64], sizeof(buffer)-64, "ip6.arpa.");
 			target.type = mDNSAddrType_IPv6;
-			bcopy(&s6, &target.ip.v6, sizeof(target.ip.v6));
+			mDNSPlatformMemCopy(&target.ip.v6, &s6, sizeof(target.ip.v6));
 			DoQuery(&q, buffer, kDNSType_PTR, &target, NameCallback);
 			if (StopNow == 2) break;
 			}
@@ -453,25 +337,24 @@ mDNSexport int main(int argc, char **argv)
 	
 		if (hardware[0] || software[0])
 			{
-			DNSQuestion q1;
 			printf("HINFO Hardware: %s\n", hardware);
 			printf("HINFO Software: %s\n", software);
-			// We need to make sure the services query is targeted
-			if (target.type == 0) target = hostaddr;
-			StartQuery(&q1, "_services._dns-sd._udp.local.", kDNSQType_ANY, &target, ServicesCallback);
-			WaitForAnswer(&mDNSStorage, 4);
-			mDNS_StopQuery(&mDNSStorage, &q1);
+			}
+		else if (NumAnswers) printf("%s has no HINFO record\n", hostname);
+		else printf("Incorrect dot-local hostname, address, or no mDNSResponder running on that machine\n");
+
+		if (NumAnswers)
+			{
+			// Because of the way we use lastsrc in ServicesCallback, we need to clear the cache to make sure we're getting fresh answers
+			mDNS *const m = &mDNSStorage;
+			mDNSu32 slot;
+			CacheGroup *cg;
+			CacheRecord *rr;
+			FORALL_CACHERECORDS(slot, cg, rr) mDNS_PurgeCacheResourceRecord(m, rr);
+			if (target.type == 0) target = hostaddr;		// Make sure the services query is targeted
+			DoQuery(&q, "_services._dns-sd._udp.local.", kDNSType_PTR, &target, ServicesCallback);
 			if (StopNow == 2) break;
 			}
-		else if (NumAnswers)
-			{
-			printf("Host has no HINFO record; Best guess is ");
-			if (id.b[1]) printf("mDNSResponder-%d\n", id.b[1]);
-			else if (NumAAAA) printf("very early Panther build (mDNSResponder-33 or earlier)\n");
-			else printf("Jaguar version of mDNSResponder with no IPv6 support\n");
-			}
-		else
-			printf("Incorrect dot-local hostname, address, or no mDNSResponder running on that machine\n");
 		}
 
 	mDNS_Close(&mDNSStorage);
