@@ -14,63 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
 
-    Change History (most recent first):
-
-$Log: JNISupport.c,v $
-Revision 1.17  2006/08/14 23:25:08  cheshire
-Re-licensed mDNSResponder daemon source code under Apache License, Version 2.0
-
-Revision 1.16  2006/07/14 02:35:47  cheshire
-Added (commented out) syslog debugging messages
-
-Revision 1.15  2006/06/27 19:34:43  cheshire
-<rdar://problem/4430023> txtRecord parameter of DNSServiceResolveReply() should be unsigned char *
-
-Revision 1.14  2006/06/20 23:03:35  rpantos
-<rdar://problem/3839132> Java needs to implement DNSServiceRegisterRecord equivalent
-
-Revision 1.13  2005/10/26 01:52:24  cheshire
-<rdar://problem/4316286> Race condition in Java code (doesn't work at all on Linux)
-
-Revision 1.12  2005/07/13 19:20:32  cheshire
-<rdar://problem/4175511> Race condition in Java API
-Additional cleanup suggested by Roger -- NewContext() doesn't need ownerClass parameter any more
-
-Revision 1.11  2005/07/11 01:55:21  cheshire
-<rdar://problem/4175511> Race condition in Java API
-
-Revision 1.10  2005/07/05 13:01:52  cheshire
-<rdar://problem/4169791> If mDNSResponder daemon is stopped, Java API spins, burning CPU time
-
-Revision 1.9  2004/12/11 03:01:00  rpantos
-<rdar://problem/3907498> Java DNSRecord API should be cleaned up
-
-Revision 1.8  2004/11/30 23:51:05  cheshire
-Remove double semicolons
-
-Revision 1.7  2004/11/23 08:12:04  shersche
-Implement if_nametoindex and if_indextoname for Win32 platforms
-
-Revision 1.6  2004/11/23 03:41:14  cheshire
-Change JNISupport.c to call if_indextoname & if_nametoindex directly.
-(May require some additional glue code to work on Windows.)
-
-Revision 1.5  2004/11/17 17:07:44  cheshire
-Updated comment about AUTO_CALLBACKS
-
-Revision 1.4  2004/11/12 03:23:09  rpantos
-rdar://problem/3809541 implement getIfIndexForName, getNameForIfIndex.
-
-Revision 1.3  2004/06/18 04:44:17  rpantos
-Adapt to API unification on Windows
-
-Revision 1.2  2004/05/28 23:34:42  ksekar
-<rdar://problem/3672903>: Java project build errors
-
-Revision 1.1  2004/04/30 16:29:35  rpantos
-First checked in.
-
-
 	This file contains the platform support for DNSSD and related Java classes.
 	It is used to shim through to the underlying <dns_sd.h> API.
  */
@@ -110,9 +53,14 @@ static DWORD	if_nametoindex( const char * nameStr );
 #include <sys/socket.h>
 #include <net/if.h>
 #endif // _WIN32
-#include <jni.h>
 
+// When compiling with "-Wshadow" set, including jni.h produces the following error:
+// /System/Library/Frameworks/JavaVM.framework/Versions/A/Headers/jni.h:609: warning: declaration of 'index' shadows a global declaration
+// To work around this, we use the preprocessor to map the identifier 'index', which appears harmlessly in function prototype declarations,
+// to something 'jni_index', which doesn't conflict
+#define index jni_index
 #include "DNSSD.java.h"
+#undef index
 
 //#include <syslog.h>
 
@@ -124,7 +72,8 @@ static DWORD	if_nametoindex( const char * nameStr );
 #endif
 
 enum {
-	kInterfaceVersion = 1		// Must match version in .jar file
+	kInterfaceVersionOne = 1,
+	kInterfaceVersionCurrent		// Must match version in .jar file
 };
 
 typedef struct OpContext	OpContext;
@@ -149,7 +98,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSSD_InitLibrary( JNIEnv *pEnv
 						jint callerVersion)
 {
 	/* Ensure that caller & interface versions match. */
-	if ( callerVersion != kInterfaceVersion)
+	if ( callerVersion != kInterfaceVersionCurrent)
 		return kDNSServiceErr_Incompatible;
 
 #if AUTO_CALLBACKS
@@ -253,15 +202,15 @@ JNIEXPORT void JNICALL Java_com_apple_dnssd_AppleService_HaltOperation( JNIEnv *
 /* Deallocate the dns_sd service browser and set the Java object's fNativeContext field to 0. */
 {
 	jclass			cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID		contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID		contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 
 	if ( contextField != 0)
 	{
-		OpContext	*pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, pThis, contextField);
+		OpContext	*pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, pThis, contextField);
 		if ( pContext != NULL)
 		{
 			// MUST clear fNativeContext first, BEFORE calling DNSServiceRefDeallocate()
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, 0);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, 0);
 			if ( pContext->ServiceRef != NULL)
 				DNSServiceRefDeallocate( pContext->ServiceRef);
 
@@ -279,11 +228,11 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleService_BlockForData( JNIEnv *p
 // BlockForData() not supported with AUTO_CALLBACKS 
 #if !AUTO_CALLBACKS
 	jclass			cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID		contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID		contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 
 	if ( contextField != 0)
 	{
-		OpContext	*pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, pThis, contextField);
+		OpContext	*pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, pThis, contextField);
 		if ( pContext != NULL)
 		{
 			fd_set			readFDs;
@@ -323,8 +272,8 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleService_ProcessResults( JNIEnv
 #if !AUTO_CALLBACKS	// ProcessResults() not supported with AUTO_CALLBACKS 
 
 	jclass			cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID		contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
-	OpContext		*pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, pThis, contextField);
+	jfieldID		contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
+	OpContext		*pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, pThis, contextField);
 	DNSServiceErrorType err = kDNSServiceErr_BadState;
 
 	if ( pContext != NULL)
@@ -387,7 +336,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleBrowser_CreateBrowser( JNIEnv *
 							jint flags, jint ifIndex, jstring regType, jstring domain)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 
@@ -409,7 +358,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleBrowser_CreateBrowser( JNIEnv *
 		err = DNSServiceBrowse( &pContext->ServiceRef, flags, ifIndex, regStr, domainStr, ServiceBrowseReply, pContext);
 		if ( err == kDNSServiceErr_NoError)
 		{
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, (jint) pContext);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, (long) pContext);
 		}
 
 		SafeReleaseUTFChars( pEnv, regType, regStr);
@@ -473,7 +422,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleResolver_CreateResolver( JNIEnv
 							jint flags, jint ifIndex, jstring serviceName, jstring regType, jstring domain)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 
@@ -493,7 +442,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleResolver_CreateResolver( JNIEnv
 								servStr, regStr, domainStr, ServiceResolveReply, pContext);
 		if ( err == kDNSServiceErr_NoError)
 		{
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, (jint) pContext);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, (long) pContext);
 		}
 
 		SafeReleaseUTFChars( pEnv, serviceName, servStr);
@@ -537,7 +486,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRegistration_BeginRegister( JNI
 {
 	//syslog(LOG_ERR, "BR");
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 	jbyte					*pBytes;
@@ -573,7 +522,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRegistration_BeginRegister( JNI
 								numBytes, pBytes, ServiceRegisterReply, pContext);
 		if ( err == kDNSServiceErr_NoError)
 		{
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, (jint) pContext);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, (long) pContext);
 		}
 
 		if ( pBytes != NULL)
@@ -594,9 +543,9 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRegistration_AddRecord( JNIEnv
 							jint flags, jint rrType, jbyteArray rData, jint ttl, jobject destObj)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	jclass					destCls = (*pEnv)->GetObjectClass( pEnv, destObj);
-	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, destCls, "fRecord", "I");
+	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, destCls, "fRecord", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 	jbyte					*pBytes;
@@ -604,7 +553,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRegistration_AddRecord( JNIEnv
 	DNSRecordRef			recRef;
 
 	if ( contextField != 0)
-		pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, pThis, contextField);
+		pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, pThis, contextField);
 	if ( pContext == NULL || pContext->ServiceRef == NULL)
 		return kDNSServiceErr_BadParam;
 
@@ -614,7 +563,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRegistration_AddRecord( JNIEnv
 	err = DNSServiceAddRecord( pContext->ServiceRef, &recRef, flags, rrType, numBytes, pBytes, ttl);
 	if ( err == kDNSServiceErr_NoError)
 	{
-		(*pEnv)->SetIntField( pEnv, destObj, recField, (jint) recRef);
+		(*pEnv)->SetLongField(pEnv, destObj, recField, (long) recRef);
 	}
 
 	if ( pBytes != NULL)
@@ -628,7 +577,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSRecord_Update( JNIEnv *pEnv,
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
 	jfieldID				ownerField = (*pEnv)->GetFieldID( pEnv, cls, "fOwner", "Lcom/apple/dnssd/AppleService;");
-	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, cls, "fRecord", "I");
+	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, cls, "fRecord", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 	jbyte					*pBytes;
@@ -639,12 +588,12 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSRecord_Update( JNIEnv *pEnv,
 	{
 		jobject		ownerObj = (*pEnv)->GetObjectField( pEnv, pThis, ownerField);
 		jclass		ownerClass = (*pEnv)->GetObjectClass( pEnv, ownerObj);
-		jfieldID	contextField = (*pEnv)->GetFieldID( pEnv, ownerClass, "fNativeContext", "I");
+		jfieldID	contextField = (*pEnv)->GetFieldID( pEnv, ownerClass, "fNativeContext", "J");
 		if ( contextField != 0)
-			pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, ownerObj, contextField);
+			pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, ownerObj, contextField);
 	}
 	if ( recField != 0)
-		recRef = (DNSRecordRef) (*pEnv)->GetIntField( pEnv, pThis, recField);
+		recRef = (DNSRecordRef) (long) (*pEnv)->GetLongField(pEnv, pThis, recField);
 	if ( pContext == NULL || pContext->ServiceRef == NULL)
 		return kDNSServiceErr_BadParam;
 
@@ -663,7 +612,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSRecord_Remove( JNIEnv *pEnv,
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
 	jfieldID				ownerField = (*pEnv)->GetFieldID( pEnv, cls, "fOwner", "Lcom/apple/dnssd/AppleService;");
-	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, cls, "fRecord", "I");
+	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, cls, "fRecord", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 	DNSRecordRef			recRef = NULL;
@@ -672,12 +621,12 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSRecord_Remove( JNIEnv *pEnv,
 	{
 		jobject		ownerObj = (*pEnv)->GetObjectField( pEnv, pThis, ownerField);
 		jclass		ownerClass = (*pEnv)->GetObjectClass( pEnv, ownerObj);
-		jfieldID	contextField = (*pEnv)->GetFieldID( pEnv, ownerClass, "fNativeContext", "I");
+		jfieldID	contextField = (*pEnv)->GetFieldID( pEnv, ownerClass, "fNativeContext", "J");
 		if ( contextField != 0)
-			pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, ownerObj, contextField);
+			pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, ownerObj, contextField);
 	}
 	if ( recField != 0)
-		recRef = (DNSRecordRef) (*pEnv)->GetIntField( pEnv, pThis, recField);
+		recRef = (DNSRecordRef) (long) (*pEnv)->GetLongField(pEnv, pThis, recField);
 	if ( pContext == NULL || pContext->ServiceRef == NULL)
 		return kDNSServiceErr_BadParam;
 
@@ -690,7 +639,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSRecord_Remove( JNIEnv *pEnv,
 JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRecordRegistrar_CreateConnection( JNIEnv *pEnv, jobject pThis)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 
@@ -704,7 +653,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRecordRegistrar_CreateConnectio
 		err = DNSServiceCreateConnection( &pContext->ServiceRef);
 		if ( err == kDNSServiceErr_NoError)
 		{
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, (jint) pContext);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, (long) pContext);
 		}
 	}
 	else
@@ -751,9 +700,9 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRecordRegistrar_RegisterRecord(
 							jbyteArray rData, jint ttl, jobject destObj)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	jclass					destCls = (*pEnv)->GetObjectClass( pEnv, destObj);
-	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, destCls, "fRecord", "I");
+	jfieldID				recField = (*pEnv)->GetFieldID( pEnv, destCls, "fRecord", "J");
 	const char				*nameStr = SafeGetUTFChars( pEnv, fullname);
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
@@ -763,7 +712,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRecordRegistrar_RegisterRecord(
 	RecordRegistrationRef	*regEnvelope;
 
 	if ( contextField != 0)
-		pContext = (OpContext*) (*pEnv)->GetIntField( pEnv, pThis, contextField);
+		pContext = (OpContext*) (long) (*pEnv)->GetLongField(pEnv, pThis, contextField);
 	if ( pContext == NULL || pContext->ServiceRef == NULL || nameStr == NULL)
 		return kDNSServiceErr_BadParam;
 
@@ -782,7 +731,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleRecordRegistrar_RegisterRecord(
 
 	if ( err == kDNSServiceErr_NoError)
 	{
-		(*pEnv)->SetIntField( pEnv, destObj, recField, (jint) recRef);
+		(*pEnv)->SetLongField(pEnv, destObj, recField, (long) recRef);
 	}
 	else
 	{
@@ -836,7 +785,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleQuery_CreateQuery( JNIEnv *pEnv
 							jint flags, jint ifIndex, jstring serviceName, jint rrtype, jint rrclass)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 
@@ -854,7 +803,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleQuery_CreateQuery( JNIEnv *pEnv
 									rrtype, rrclass, ServiceQueryReply, pContext);
 		if ( err == kDNSServiceErr_NoError)
 		{
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, (jint) pContext);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, (long) pContext);
 		}
 
 		SafeReleaseUTFChars( pEnv, serviceName, servStr);
@@ -892,7 +841,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDomainEnum_BeginEnum( JNIEnv *p
 							jint flags, jint ifIndex)
 {
 	jclass					cls = (*pEnv)->GetObjectClass( pEnv, pThis);
-	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "I");
+	jfieldID				contextField = (*pEnv)->GetFieldID( pEnv, cls, "fNativeContext", "J");
 	OpContext				*pContext = NULL;
 	DNSServiceErrorType		err = kDNSServiceErr_NoError;
 
@@ -912,7 +861,7 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDomainEnum_BeginEnum( JNIEnv *p
 											DomainEnumReply, pContext);
 		if ( err == kDNSServiceErr_NoError)
 		{
-			(*pEnv)->SetIntField( pEnv, pThis, contextField, (jint) pContext);
+			(*pEnv)->SetLongField(pEnv, pThis, contextField, (long) pContext);
 		}
 	}
 	else
@@ -966,13 +915,16 @@ JNIEXPORT void JNICALL Java_com_apple_dnssd_AppleDNSSD_ReconfirmRecord( JNIEnv *
 }
 
 #define LOCAL_ONLY_NAME "loo"
+#define P2P_NAME "p2p"
 
 JNIEXPORT jstring JNICALL Java_com_apple_dnssd_AppleDNSSD_GetNameForIfIndex( JNIEnv *pEnv, jobject pThis _UNUSED,
 							jint ifIndex)
 {
 	char					*p = LOCAL_ONLY_NAME, nameBuff[IF_NAMESIZE];
 
-	if (ifIndex != (jint) kDNSServiceInterfaceIndexLocalOnly)
+	if (ifIndex == (jint) kDNSServiceInterfaceIndexP2P)
+		p = P2P_NAME;
+	else if (ifIndex != (jint) kDNSServiceInterfaceIndexLocalOnly)
 		p = if_indextoname( ifIndex, nameBuff );
 
 	return (*pEnv)->NewStringUTF( pEnv, p);
@@ -985,7 +937,9 @@ JNIEXPORT jint JNICALL Java_com_apple_dnssd_AppleDNSSD_GetIfIndexForName( JNIEnv
 	uint32_t				ifIndex = kDNSServiceInterfaceIndexLocalOnly;
 	const char				*nameStr = SafeGetUTFChars( pEnv, ifName);
 
-	if (strcmp(nameStr, LOCAL_ONLY_NAME))
+	if (strcmp(nameStr, P2P_NAME) == 0)
+		ifIndex = kDNSServiceInterfaceIndexP2P;
+	else if (strcmp(nameStr, LOCAL_ONLY_NAME))
 		ifIndex = if_nametoindex( nameStr);
 
 	SafeReleaseUTFChars( pEnv, ifName, nameStr);
@@ -1103,3 +1057,14 @@ exit:
 	return ifIndex;
 }
 #endif
+
+
+// Note: The C preprocessor stringify operator ('#') makes a string from its argument, without macro expansion
+// e.g. If "version" is #define'd to be "4", then STRINGIFY_AWE(version) will return the string "version", not "4"
+// To expand "version" to its value before making the string, use STRINGIFY(version) instead
+#define STRINGIFY_ARGUMENT_WITHOUT_EXPANSION(s) #s
+#define STRINGIFY(s) STRINGIFY_ARGUMENT_WITHOUT_EXPANSION(s)
+
+// NOT static -- otherwise the compiler may optimize it out
+// The "@(#) " pattern is a special prefix the "what" command looks for
+const char VersionString_SCCS[] = "@(#) libjdns_sd " STRINGIFY(mDNSResponderVersion) " (" __DATE__ " " __TIME__ ")";
